@@ -39,7 +39,7 @@ func tsrStatefulSet(tsr *tsapi.Recorder, namespace string) *appsv1.StatefulSet {
 					Annotations: tsr.Spec.StatefulSet.Pod.Annotations,
 				},
 				Spec: corev1.PodSpec{
-					ServiceAccountName: tsr.Name,
+					ServiceAccountName: tsrServiceAccountName(tsr),
 					Affinity:           tsr.Spec.StatefulSet.Pod.Affinity,
 					SecurityContext:    tsr.Spec.StatefulSet.Pod.SecurityContext,
 					ImagePullSecrets:   tsr.Spec.StatefulSet.Pod.ImagePullSecrets,
@@ -100,14 +100,25 @@ func tsrStatefulSet(tsr *tsapi.Recorder, namespace string) *appsv1.StatefulSet {
 func tsrServiceAccount(tsr *tsapi.Recorder, namespace string) *corev1.ServiceAccount {
 	return &corev1.ServiceAccount{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:            tsr.Name,
+			Name:            tsrServiceAccountName(tsr),
 			Namespace:       namespace,
 			Labels:          labels("recorder", tsr.Name, nil),
 			OwnerReferences: tsrOwnerReference(tsr),
+			Annotations:     tsr.Spec.StatefulSet.Pod.ServiceAccount.Annotations,
 		},
 	}
 }
 
+func tsrServiceAccountName(tsr *tsapi.Recorder) string {
+	sa := tsr.Spec.StatefulSet.Pod.ServiceAccount
+	name := tsr.Name
+	if sa.Name != "" {
+		name = sa.Name
+	}
+
+	return name
+}
+
 func tsrRole(tsr *tsapi.Recorder, namespace string) *rbacv1.Role {
 	return &rbacv1.Role{
 		ObjectMeta: metav1.ObjectMeta{
@@ -154,7 +165,7 @@ func tsrRoleBinding(tsr *tsapi.Recorder, namespace string) *rbacv1.RoleBinding {
 		Subjects: []rbacv1.Subject{
 			{
 				Kind:      "ServiceAccount",
-				Name:      tsr.Name,
+				Name:      tsrServiceAccountName(tsr),
 				Namespace: namespace,
 			},
 		},
