@@ -44,6 +44,7 @@ import (
 	"tailscale.com/net/dnscache"
 	"tailscale.com/net/dnsfallback"
 	"tailscale.com/net/netutil"
+	"tailscale.com/net/netx"
 	"tailscale.com/net/sockstats"
 	"tailscale.com/net/tlsdial"
 	"tailscale.com/net/tshttpproxy"
@@ -249,6 +250,11 @@ func (a *Dialer) dial(ctx context.Context) (*ClientConn, error) {
 		results[i].conn = nil // so we don't close it in the defer
 		return conn, nil
 	}
+	if ctx.Err() != nil {
+		a.logf("controlhttp: context aborted dialing")
+		return nil, ctx.Err()
+	}
+
 	merr := multierr.New(errs...)
 
 	// If we get here, then we didn't get anywhere with our dial plan; fall back to just using DNS.
@@ -270,6 +276,15 @@ var forceNoise443 = envknob.RegisterBool("TS_FORCE_NOISE_443")
 // use HTTPS connections as its underlay connection (double crypto). This can
 // be necessary when networks or middle boxes are messing with port 80.
 func (d *Dialer) forceNoise443() bool {
+	if runtime.GOOS == "plan9" {
+		// For running demos of Plan 9 in a browser with network relays,
+		// we want to minimize the number of connections we're making.
+		// The main reason to use port 80 is to avoid double crypto
+		// costs server-side but the costs are tiny and number of Plan 9
+		// users doesn't make it worth it. Just disable this and always use
+		// HTTPS for Plan 9. That also reduces some log spam.
+		return true
+	}
 	if forceNoise443() {
 		return true
 	}
@@ -480,7 +495,7 @@ func (a *Dialer) tryURLUpgrade(ctx context.Context, u *url.URL, optAddr netip.Ad
 		dns = a.resolver()
 	}
 
-	var dialer dnscache.DialContextFunc
+	var dialer netx.DialFunc
 	if a.Dialer != nil {
 		dialer = a.Dialer
 	} else {
