@@ -1,7 +1,7 @@
 // Copyright (c) Tailscale Inc & AUTHORS
 // SPDX-License-Identifier: BSD-3-Clause
 
-//go:build (ts_aws || (linux && (arm64 || amd64))) && !ts_omit_aws
+//go:build (ts_aws || (linux && (arm64 || amd64) && !android)) && !ts_omit_aws
 
 package store
 
@@ -12,10 +12,6 @@ import (
 )
 
 func init() {
-	registerAvailableExternalStores = append(registerAvailableExternalStores, registerAWSStore)
-}
-
-func registerAWSStore() {
 	Register("arn:", func(logf logger.Logf, arg string) (ipn.StateStore, error) {
 		ssmARN, opts, err := awsstore.ParseARNAndOpts(arg)
 		if err != nil {
